@@ -2,11 +2,12 @@ use crate::{
     allocator::Allocator,
     config::Config,
     discref::{Page, PageOps, PAGE_METADATA_SIZE},
-    error::{PRes, PersyError},
+    error::{GenericError, PERes, ReadError, SegmentError, TimeoutError},
     id::{PersyId, RecRef, SegmentId},
     locks::{LockManager, RwLockManager},
     segment::{AllocatedSegmentPage, SegmentPage, SegmentPageIterator, SegmentPageRead, Segments},
-    transaction::{DeleteRecord, InsertRecord, NewSegmentPage, SegmentOperation, UpdateRecord},
+    transaction_impl::{DeleteRecord, InsertRecord, NewSegmentPage, SegmentOperation, UpdateRecord},
+    PrepareError,
 };
 use std::{
     collections::{hash_map::Entry, HashMap, HashSet},
@@ -53,7 +54,7 @@ pub struct Address {
 }
 
 impl Address {
-    pub fn new(all: &Arc<Allocator>, config: &Arc<Config>, page: u64) -> PRes<Address> {
+    pub fn new(all: &Arc<Allocator>, config: &Arc<Config>, page: u64) -> PERes<Address> {
         let segments = Segments::new(page, all)?;
         Ok(Address {
             config: config.clone(),
@@ -64,52 +65,63 @@ impl Address {
         })
     }
 
-    pub fn init(all: &Allocator) -> PRes<u64> {
+    pub fn init(all: &Allocator) -> PERes<u64> {
         let page = all.allocate(ADDRESS_ROOT_PAGE_EXP)?;
         let page_index = page.get_index();
         Segments::init(page, all)?;
         Ok(page_index)
     }
 
-    pub fn scan(&self, segment: SegmentId) -> PRes<SegmentPageIterator> {
-        let segments = self.segments.read()?;
+    pub fn scan(&self, segment: SegmentId) -> Result<SegmentPageIterator, SegmentError> {
+        let segments = self.segments.read().map_err(GenericError::from)?;
         if let Some(segment) = segments.segment_by_id(segment) {
             Ok(SegmentPageIterator::new(segment.first_page))
         } else if let Some(temp_segment) = segments.segment_by_id_temp(segment) {
             Ok(SegmentPageIterator::new(temp_segment.first_page))
         } else {
-            Err(PersyError::SegmentNotFound)
+            Err(SegmentError::SegmentNotFound)
         }
     }
 
-    pub fn scan_page_all(&self, cur_page: u64) -> PRes<(u64, [(u32, bool); ADDRESS_PAGE_ENTRY_COUNT as usize])> {
+    pub fn scan_page_all(&self, cur_page: u64) -> PERes<(u64, [(u32, bool); ADDRESS_PAGE_ENTRY_COUNT as usize])> {
         // THIS IS ONLY FOR LOCK PROTECTION
         let _lock = self.segments.read()?;
         let mut page = self.allocator.load_page(cur_page)?;
-        page.segment_scan_all_entries()
+        Ok(page.segment_scan_all_entries())
     }
 
-    pub fn allocate_temp(&self, segment: SegmentId) -> PRes<(RecRef, Option<AllocatedSegmentPage>)> {
-        if let Some(found) = self.segments.write()?.get_temp_segment_mut(segment) {
-            found.allocate_internal(&self.allocator)
+    pub fn allocate_temp(&self, segment: SegmentId) -> Result<(RecRef, Option<AllocatedSegmentPage>), SegmentError> {
+        if let Some(found) = self
+            .segments
+            .write()
+            .map_err(GenericError::from)?
+            .get_temp_segment_mut(segment)
+        {
+            Ok(found.allocate_internal(&self.allocator)?)
         } else {
-            Err(PersyError::SegmentNotFound)
+            Err(SegmentError::SegmentNotFound)
         }
     }
 
-    pub fn create_temp_segment(&self, segment: &str) -> PRes<(SegmentId, u64)> {
+    pub fn create_temp_segment(&self, segment: &str) -> PERes<(SegmentId, u64)> {
         self.segments.write()?.create_temp_segment(&self.allocator, segment)
     }
 
-    pub fn drop_temp_segment(&self, segment: SegmentId) -> PRes<()> {
+    pub fn drop_temp_segment(&self, segment: SegmentId) -> PERes<()> {
         self.segments.write()?.drop_temp_segment(&self.allocator, segment)
     }
 
-    pub fn allocate(&self, segment: SegmentId) -> PRes<(RecRef, Option<AllocatedSegmentPage>)> {
-        if let Some(found) = self.segments.write()?.segments.get_mut(&segment) {
-            found.allocate_internal(&self.allocator)
+    pub fn allocate(&self, segment: SegmentId) -> Result<(RecRef, Option<AllocatedSegmentPage>), SegmentError> {
+        if let Some(found) = self
+            .segments
+            .write()
+            .map_err(GenericError::from)?
+            .segments
+            .get_mut(&segment)
+        {
+            Ok(found.allocate_internal(&self.allocator)?)
         } else {
-            Err(PersyError::SegmentNotFound)
+            Err(SegmentError::SegmentNotFound)
         }
     }
 
@@ -118,68 +130,68 @@ impl Address {
         records: &[(SegmentId, RecRef, u16)],
         created_updated: &[SegmentId],
         deleted: &[SegmentId],
-    ) -> PRes<()> {
+    ) -> Result<(), PrepareError> {
         let timeout = *self.config.transaction_lock_timeout();
-        self.segment_locks.lock_all_write(&deleted, timeout)?;
-        if let Err(x) = self.segment_locks.lock_all_read(&created_updated, timeout) {
-            if let Err(e) = self.segment_locks.unlock_all_write(&deleted) {
+        self.segment_locks.lock_all_write(deleted, timeout)?;
+        if let Err(x) = self.segment_locks.lock_all_read(created_updated, timeout) {
+            if let Err(e) = self.segment_locks.unlock_all_write(deleted) {
                 dbg!("unlock error: {}", e);
             }
-            return Err(x);
+            return Err(PrepareError::from(x));
         }
 
         let to_lock: Vec<_> = records.iter().map(|(_, id, _)| id.clone()).collect();
         if let Err(x) = self.record_locks.lock_all(&to_lock, timeout) {
-            if let Err(e) = self.segment_locks.unlock_all_write(&deleted) {
+            if let Err(e) = self.segment_locks.unlock_all_write(deleted) {
                 dbg!("unlock error: {}", e);
             }
-            if let Err(e) = self.segment_locks.unlock_all_read(&created_updated) {
+            if let Err(e) = self.segment_locks.unlock_all_read(created_updated) {
                 dbg!("unlock error: {}", e);
             }
-            return Err(x);
+            return Err(PrepareError::from(x));
         }
 
         let segs = self.segments.read()?;
         for segment in created_updated {
             if !segs.exists_real_or_temp(*segment) {
-                if let Err(e) = self.segment_locks.unlock_all_write(&deleted) {
+                if let Err(e) = self.segment_locks.unlock_all_write(deleted) {
                     dbg!("unlock error: {}", e);
                 }
-                if let Err(e) = self.segment_locks.unlock_all_read(&created_updated) {
+                if let Err(e) = self.segment_locks.unlock_all_read(created_updated) {
                     dbg!("unlock error: {}", e);
                 }
                 if let Err(e) = self.record_locks.unlock_all(&to_lock) {
                     dbg!("unlock error: {}", e);
                 }
-                return Err(PersyError::SegmentNotFound);
+                return Err(PrepareError::SegmentNotFound);
             }
         }
         Ok(())
     }
 
-    pub fn acquire_segment_read_lock(&self, segment: SegmentId) -> PRes<()> {
+    pub fn acquire_segment_read_lock(&self, segment: SegmentId) -> Result<(), TimeoutError> {
         let timeout = *self.config.transaction_lock_timeout();
         self.segment_locks.lock_all_read(&[segment], timeout)?;
         Ok(())
     }
-    pub fn acquire_record_lock(&self, id: &RecRef) -> PRes<()> {
+    pub fn acquire_record_lock(&self, id: &RecRef) -> Result<(), TimeoutError> {
         let timeout = *self.config.transaction_lock_timeout();
         self.record_locks.lock_all(&[id.clone()], timeout)?;
         Ok(())
     }
 
-    pub fn release_segment_read_lock(&self, segment: SegmentId) -> PRes<()> {
+    pub fn release_segment_read_lock(&self, segment: SegmentId) -> PERes<()> {
         self.segment_locks.unlock_all_read(&[segment])?;
         Ok(())
     }
-    pub fn release_record_lock(&self, id: &RecRef) -> PRes<()> {
+    pub fn release_record_lock(&self, id: &RecRef) -> PERes<()> {
         self.record_locks.unlock_all(&[id.clone()])?;
         Ok(())
     }
 
-    pub fn confirm_allocations(&self, segs: &[SegmentId], recover: bool) -> PRes<()> {
+    pub fn confirm_allocations(&self, segs: &[SegmentId], recover: bool) -> PERes<()> {
         let mut segments = self.segments.write()?;
-        segments.confirm_allocations(&segs, &self.allocator, recover)?;
+        segments.confirm_allocations(segs, &self.allocator, recover)?;
         Ok(())
     }
 
@@ -187,17 +199,17 @@ impl Address {
         &self,
         records: &[(SegmentId, RecRef, u16)],
         check_version: bool,
-    ) -> PRes<Vec<OldRecordInfo>> {
+    ) -> Result<Vec<OldRecordInfo>, PrepareError> {
         let mut current_record_pages = Vec::with_capacity(records.len());
         for &(segment, ref recref, version) in records {
             let val = self.read(recref, segment)?;
             if let Some((record, pers_version)) = val {
-                current_record_pages.push(OldRecordInfo::new(&recref, segment, record, pers_version));
+                current_record_pages.push(OldRecordInfo::new(recref, segment, record, pers_version));
                 if check_version && pers_version != version {
-                    return Err(PersyError::VersionNotLastest);
+                    return Err(PrepareError::VersionNotLastest);
                 }
             } else {
-                return Err(PersyError::RecordNotFound(PersyId(recref.clone())));
+                return Err(PrepareError::RecordNotFound(PersyId(recref.clone())));
             }
         }
         Ok(current_record_pages)
@@ -208,14 +220,14 @@ impl Address {
         records: impl Iterator<Item = &'a RecRef>,
         created_updated: &[SegmentId],
         deleted: &[SegmentId],
-    ) -> PRes<()> {
+    ) -> PERes<()> {
         self.record_locks.unlock_all_iter(records)?;
-        self.segment_locks.unlock_all_read(&created_updated)?;
-        self.segment_locks.unlock_all_write(&deleted)?;
+        self.segment_locks.unlock_all_read(created_updated)?;
+        self.segment_locks.unlock_all_write(deleted)?;
         Ok(())
     }
 
-    pub fn rollback(&self, inserts: &[InsertRecord]) -> PRes<Vec<(SegmentId, u64)>> {
+    pub fn rollback(&self, inserts: &[InsertRecord]) -> PERes<Vec<(SegmentId, u64)>> {
         let segments = self.segments.write()?;
         let mut pages_to_remove = Vec::new();
         let mut pages = HashMap::new();
@@ -223,7 +235,7 @@ impl Address {
             if segments.segments.contains_key(&insert.segment) {
                 let page = insert.recref.page;
                 let seg_page = self.get_or_insert_mut(&mut pages, page)?;
-                if seg_page.segment_delete_entry(insert.segment, insert.recref.pos)? && seg_page.get_next()? != 0 {
+                if seg_page.segment_delete_entry(insert.segment, insert.recref.pos) && seg_page.get_next()? != 0 {
                     pages_to_remove.push((insert.segment, page));
                 }
             }
@@ -242,11 +254,11 @@ impl Address {
         deletes: &[DeleteRecord],
         seg_ops: &[SegmentOperation],
         recover: bool,
-    ) -> PRes<Vec<(SegmentId, u64)>> {
+    ) -> PERes<Vec<(SegmentId, u64)>> {
         let mut segments = self.segments.write()?;
         let mut dropped = HashSet::new();
         for seg_op in seg_ops {
-            if let SegmentOperation::DROP(ref op) = *seg_op {
+            if let SegmentOperation::Drop(ref op) = *seg_op {
                 dropped.insert(op.segment_id);
             }
         }
@@ -265,7 +277,7 @@ impl Address {
             if !dropped.contains(&insert.segment) {
                 let page = insert.recref.page;
                 let seg_page = self.get_or_insert_mut(&mut pages, page)?;
-                seg_page.segment_insert_entry(insert.segment, insert.recref.pos, insert.record_page)?;
+                seg_page.segment_insert_entry(insert.segment, insert.recref.pos, insert.record_page);
             }
         }
 
@@ -273,7 +285,7 @@ impl Address {
             if !dropped.contains(&update.segment) {
                 let page = update.recref.page;
                 let seg_page = self.get_or_insert_mut(&mut pages, page)?;
-                seg_page.segment_update_entry(update.segment, update.recref.pos, update.record_page)?;
+                seg_page.segment_update_entry(update.segment, update.recref.pos, update.record_page);
             }
         }
         let mut pages_to_remove = Vec::new();
@@ -282,7 +294,7 @@ impl Address {
             if !dropped.contains(&delete.segment) {
                 let page = delete.recref.page;
                 let seg_page = self.get_or_insert_mut(&mut pages, page)?;
-                if seg_page.segment_delete_entry(delete.segment, delete.recref.pos)? {
+                if seg_page.segment_delete_entry(delete.segment, delete.recref.pos) {
                     // Avoid to remove last pages, for avoid concurrent operations with page
                     // creation
                     if seg_page.get_next()? != 0 {
@@ -323,14 +335,14 @@ impl Address {
         }
 
         for seg_op in seg_ops {
-            if let SegmentOperation::DROP(ref op) = *seg_op {
-                segments.drop_segment(&op.name)?;
+            if let SegmentOperation::Drop(ref op) = *seg_op {
+                segments.drop_segment(&op.name);
             }
         }
 
         for seg_op in seg_ops {
-            if let SegmentOperation::CREATE(ref op) = *seg_op {
-                segments.create_segment(op.segment_id, op.first_page)?;
+            if let SegmentOperation::Create(ref op) = *seg_op {
+                segments.create_segment(op.segment_id, op.first_page);
             }
         }
         segments.flush_segments(&self.allocator)?;
@@ -338,12 +350,12 @@ impl Address {
         Ok(pages_to_remove)
     }
 
-    pub fn collect_segment_pages(&self, segment: SegmentId) -> PRes<Vec<u64>> {
+    pub fn collect_segment_pages(&self, segment: SegmentId) -> PERes<Vec<u64>> {
         let segments = self.segments.read()?;
         segments.collect_segment_pages(&self.allocator, segment)
     }
 
-    pub fn clear_empty(&self, empty: &[(SegmentId, u64)]) -> PRes<()> {
+    pub fn clear_empty(&self, empty: &[(SegmentId, u64)]) -> PERes<()> {
         let mut segments = self.segments.write()?;
         let mut flush_segments = false;
         for (segment, page) in empty {
@@ -371,46 +383,49 @@ impl Address {
         Ok(())
     }
 
-    pub fn exists_segment(&self, segment: &str) -> PRes<bool> {
+    pub fn exists_segment(&self, segment: &str) -> PERes<bool> {
         Ok(self.segments.read()?.has_segment(segment))
     }
 
-    pub fn segment_id(&self, segment: &str) -> PRes<Option<SegmentId>> {
+    pub fn segment_id(&self, segment: &str) -> PERes<Option<SegmentId>> {
         Ok(self.segments.read()?.segment_id(segment))
     }
 
-    pub fn segment_name_by_id(&self, segment: SegmentId) -> PRes<Option<String>> {
+    pub fn segment_name_by_id(&self, segment: SegmentId) -> PERes<Option<String>> {
         Ok(self.segments.read()?.segment_name_by_id(segment))
     }
 
     // Used only from the tests
     #[allow(dead_code)]
-    pub fn insert(&self, segment_id: SegmentId, recref: &RecRef, record_page: u64) -> PRes<()> {
+    pub fn insert(&self, segment_id: SegmentId, recref: &RecRef, record_page: u64) -> PERes<()> {
         let mut page = self.allocator.write_page(recref.page)?;
-        page.segment_insert_entry(segment_id, recref.pos, record_page)?;
+        page.segment_insert_entry(segment_id, recref.pos, record_page);
         self.allocator.flush_page(page)?;
         Ok(())
     }
 
-    pub fn read(&self, recref: &RecRef, segment: SegmentId) -> PRes<Option<(u64, u16)>> {
+    pub fn read(&self, recref: &RecRef, segment: SegmentId) -> Result<Option<(u64, u16)>, ReadError> {
         if let Some(mut page) = self.allocator.load_page_not_free(recref.page)? {
-            Ok(page.segment_read_entry(segment, recref.pos)?)
+            if recref.pos > ADDRESS_PAGE_SIZE - ADDRESS_ENTRY_SIZE {
+                return Err(ReadError::InvalidPersyId(recref.clone()));
+            }
+            Ok(page.segment_read_entry(segment, recref.pos))
         } else {
             Ok(None)
         }
     }
 
-    fn get_or_insert_mut<'a>(&self, map: &'a mut HashMap<u64, Page>, k: u64) -> PRes<&'a mut Page> {
+    fn get_or_insert_mut<'a>(&self, map: &'a mut HashMap<u64, Page>, k: u64) -> PERes<&'a mut Page> {
         Ok(match map.entry(k) {
             Entry::Occupied(entry) => entry.into_mut(),
             Entry::Vacant(entry) => entry.insert(self.allocator.write_page(k)?),
         })
     }
 
-    pub fn list(&self) -> PRes<Vec<(String, SegmentId)>> {
+    pub fn list(&self) -> PERes<Vec<(String, SegmentId)>> {
         Ok(self.segments.read()?.list())
     }
-    pub fn snapshot_list(&self) -> PRes<Vec<(String, SegmentId, u64)>> {
+    pub fn snapshot_list(&self) -> PERes<Vec<(String, SegmentId, u64)>> {
         Ok(self.segments.read()?.snapshot_list())
     }
 }
@@ -419,7 +434,7 @@ impl Address {
 mod tests {
     use super::Address;
     use crate::id::SegmentId;
-    use crate::transaction::{CreateSegment, DeleteRecord, InsertRecord, SegmentOperation, UpdateRecord};
+    use crate::transaction_impl::{CreateSegment, DeleteRecord, InsertRecord, SegmentOperation, UpdateRecord};
     use crate::{allocator::Allocator, config::Config, discref::DiscRef};
     use std::sync::Arc;
     use tempfile::Builder;
@@ -432,7 +447,7 @@ mod tests {
         let page = Address::init(&allocator).unwrap();
         let addr = Address::new(&Arc::new(allocator), &config, page).unwrap();
         let (id, fp) = addr.create_temp_segment("def").unwrap();
-        addr.segments.write().unwrap().create_segment(id, fp).unwrap();
+        addr.segments.write().unwrap().create_segment(id, fp);
         (addr, id)
     }
 
@@ -473,7 +488,7 @@ mod tests {
 
         deleted.push(DeleteRecord::new(segment_id, &recref, 1));
         let mut seg_ops = Vec::new();
-        seg_ops.push(SegmentOperation::CREATE(CreateSegment::new(
+        seg_ops.push(SegmentOperation::Create(CreateSegment::new(
             "def",
             SegmentId::new(20),
             20,
