@@ -1,8 +1,11 @@
 use super::{
-    Index, IndexApply, IndexKeeper, IndexModify, IndexType, KeyChanges, Leaf, Node, NodeRef, Nodes, PRes, Value,
-    ValueChange, ValueMode,
+    Index, IndexApply, IndexKeeper, IndexModify, IndexType, KeyChanges, Leaf, Node, NodeRef, Nodes, Value, ValueChange,
+    ValueMode,
+};
+use crate::{
+    error::{IndexChangeError, PERes, PIRes},
+    id::RecRef,
 };
-use crate::{error::PersyError, id::RecRef};
 use rand::random;
 use std::{collections::HashMap, ops::Bound, rc::Rc};
 
@@ -12,14 +15,14 @@ where
 {
     fn fmt(&self, f: &mut std::fmt::Formatter) -> std::fmt::Result {
         match self {
-            Value::CLUSTER(x) => {
+            Value::Cluster(x) => {
                 write!(f, "{} values: [", x.len())?;
                 for v in x {
                     write!(f, " {}, ", v)?;
                 }
                 write!(f, "]")?;
             }
-            Value::SINGLE(v) => {
+            Value::Single(v) => {
                 write!(f, "value: {}", v)?;
             }
         }
@@ -31,7 +34,7 @@ fn print_nodes<K: IndexType, V: IndexType>(
     tree: &mut dyn IndexKeeper<K, V>,
     node: &Nodes<K>,
     depth: usize,
-) -> PRes<()> {
+) -> PERes<()> {
     let padding = String::from_utf8(vec![b' '; depth]).unwrap();
     for i in 0..node.len() {
         if i == 0 {
@@ -48,7 +51,7 @@ fn print_leaf<K: IndexType, V: IndexType>(
     _tree: &mut dyn IndexKeeper<K, V>,
     leaf: &Leaf<K, V>,
     depth: usize,
-) -> PRes<()> {
+) -> PERes<()> {
     let padding = String::from_utf8(vec![b' '; depth]).unwrap();
     println!("{} ", padding);
     for i in 0..leaf.len() {
@@ -58,19 +61,19 @@ fn print_leaf<K: IndexType, V: IndexType>(
     Ok(())
 }
 
-fn print_node<K: IndexType, V: IndexType>(tree: &mut dyn IndexKeeper<K, V>, node: &NodeRef, depth: usize) -> PRes<()> {
+fn print_node<K: IndexType, V: IndexType>(tree: &mut dyn IndexKeeper<K, V>, node: &NodeRef, depth: usize) -> PERes<()> {
     match tree.load(node)? {
-        Node::NODE(n) => {
+        Node::Node(n) => {
             print_nodes(tree, &n, depth)?;
         }
-        Node::LEAF(l) => {
+        Node::Leaf(l) => {
             print_leaf(tree, &l, depth)?;
         }
     }
     Ok(())
 }
 
-fn print_tree<K: IndexType, V: IndexType>(tree: &mut dyn IndexKeeper<K, V>) -> PRes<()> {
+fn print_tree<K: IndexType, V: IndexType>(tree: &mut dyn IndexKeeper<K, V>) -> PERes<()> {
     let root = tree.get_root()?;
     if let Some(r) = root {
         print_node(tree, &r, 0)?;
@@ -96,7 +99,7 @@ impl<K: Clone + Ord, V: Clone> MockIndexKeeper<K, V> {
         MockIndexKeeper {
             store: HashMap::new(),
             root: None,
-            v: ValueMode::REPLACE,
+            v: ValueMode::Replace,
             name: "test_index".to_string(),
         }
     }
@@ -112,10 +115,10 @@ impl<K: Clone + Ord, V: Clone> MockIndexKeeper<K, V> {
 }
 
 impl<K: Clone + Ord, V: Clone> IndexKeeper<K, V> for MockIndexKeeper<K, V> {
-    fn load(&self, node: &NodeRef) -> PRes<Node<K, V>> {
+    fn load(&self, node: &NodeRef) -> PERes<Node<K, V>> {
         Ok(self.store.get(&node).unwrap().clone())
     }
-    fn get_root(&self) -> PRes<Option<NodeRef>> {
+    fn get_root(&self) -> PERes<Option<NodeRef>> {
         Ok(self.root.clone())
     }
 
@@ -129,22 +132,22 @@ impl<K: Clone + Ord, V: Clone> IndexKeeper<K, V> for MockIndexKeeper<K, V> {
 }
 
 impl<K: Clone + Ord, V: Clone> IndexModify<K, V> for MockIndexKeeper<K, V> {
-    fn load_modify(&self, node: &NodeRef) -> PRes<Option<(Rc<Node<K, V>>, u16)>> {
+    fn load_modify(&self, node: &NodeRef) -> PIRes<Option<(Rc<Node<K, V>>, u16)>> {
         Ok(Some((Rc::new(self.store.get(&node).unwrap().clone()), 0)))
     }
 
-    fn lock(&mut self, _node: &NodeRef, _version: u16) -> PRes<bool> {
+    fn lock(&mut self, _node: &NodeRef, _version: u16) -> PIRes<bool> {
         Ok(true)
     }
 
-    fn unlock(&mut self, _node: &NodeRef) -> PRes<bool> {
+    fn unlock(&mut self, _node: &NodeRef) -> PIRes<bool> {
         Ok(true)
     }
-    fn unlock_config(&mut self) -> PRes<bool> {
+    fn unlock_config(&mut self) -> PIRes<bool> {
         Ok(true)
     }
 
-    fn lock_config(&mut self) -> PRes<bool> {
+    fn lock_config(&mut self) -> PIRes<bool> {
         Ok(true)
     }
 
@@ -152,26 +155,26 @@ impl<K: Clone + Ord, V: Clone> IndexModify<K, V> for MockIndexKeeper<K, V> {
         Rc::make_mut(&mut node);
         Rc::try_unwrap(node).ok().unwrap()
     }
-    fn insert(&mut self, node: Node<K, V>) -> PRes<NodeRef> {
+    fn insert(&mut self, node: Node<K, V>) -> PIRes<NodeRef> {
         let node_ref = random_pointer();
         self.store.insert(node_ref.clone(), node.clone());
         Ok(node_ref)
     }
 
-    fn update(&mut self, node_ref: &NodeRef, node: Node<K, V>, _version: u16) -> PRes<()> {
+    fn update(&mut self, node_ref: &NodeRef, node: Node<K, V>, _version: u16) -> PIRes<()> {
         self.store.insert(node_ref.clone(), node);
         Ok(())
     }
 
-    fn get_root_refresh(&mut self) -> PRes<Option<NodeRef>> {
+    fn get_root_refresh(&mut self) -> PIRes<Option<NodeRef>> {
         Ok(self.root.clone())
     }
 
-    fn set_root(&mut self, root: Option<NodeRef>) -> PRes<()> {
+    fn set_root(&mut self, root: Option<NodeRef>) -> PIRes<()> {
         Ok(self.root = root)
     }
 
-    fn delete(&mut self, node: &NodeRef, _version: u16) -> PRes<()> {
+    fn delete(&mut self, node: &NodeRef, _version: u16) -> PIRes<()> {
         self.store.remove(&node);
         Ok(())
     }
@@ -191,7 +194,7 @@ fn test_single_add() {
     changes.push(KeyChanges::single_add(2, 2));
     changes.push(KeyChanges::single_add(3, 4));
     keeper.apply(&changes).unwrap();
-    assert_eq!(keeper.get(&2).ok(), Some(Some(Value::SINGLE(2))));
+    assert_eq!(keeper.get(&2).ok(), Some(Some(Value::Single(2))));
 }
 
 #[test]
@@ -202,8 +205,8 @@ fn test_many_add() {
         changes.push(KeyChanges::single_add(i, i));
     }
     keeper.apply(&changes).unwrap();
-    assert_eq!(keeper.get(&2).ok(), Some(Some(Value::SINGLE(2))));
-    assert_eq!(keeper.get(&100).ok(), Some(Some(Value::SINGLE(100))));
+    assert_eq!(keeper.get(&2).ok(), Some(Some(Value::Single(2))));
+    assert_eq!(keeper.get(&100).ok(), Some(Some(Value::Single(100))));
     assert_eq!(keeper.get(&201).ok(), Some(None));
 }
 
@@ -216,7 +219,7 @@ fn test_many_add_multiple_times() {
             changes.push(KeyChanges::single_add(i, i));
         }
         keeper.apply(&changes).unwrap();
-        assert_eq!(keeper.get(&(n + 2)).ok(), Some(Some(Value::SINGLE(n + 2))));
+        assert_eq!(keeper.get(&(n + 2)).ok(), Some(Some(Value::Single(n + 2))));
     }
 }
 
@@ -228,7 +231,7 @@ fn test_single_add_remove() {
     changes.push(KeyChanges::single_add(2, 2));
     changes.push(KeyChanges::single_add(3, 4));
     keeper.apply(&changes).unwrap();
-    assert_eq!(keeper.get(&2).ok(), Some(Some(Value::SINGLE(2))));
+    assert_eq!(keeper.get(&2).ok(), Some(Some(Value::Single(2))));
 
     let mut changes = Vec::new();
     changes.push(KeyChanges::single_delete(1, Some(1)));
@@ -244,18 +247,18 @@ fn test_aggregate_add_remove() {
     let mut changes = Vec::new();
     changes.push(KeyChanges::new(
         2,
-        vec![ValueChange::ADD(1), ValueChange::REMOVE(Some(1)), ValueChange::ADD(2)],
+        vec![ValueChange::Add(1), ValueChange::Remove(Some(1)), ValueChange::Add(2)],
     ));
     keeper.apply(&changes).unwrap();
-    assert_eq!(keeper.get(&2).ok(), Some(Some(Value::SINGLE(2))));
+    assert_eq!(keeper.get(&2).ok(), Some(Some(Value::Single(2))));
 
     let mut changes = Vec::new();
     changes.push(KeyChanges::new(
         2,
         vec![
-            ValueChange::REMOVE(Some(2)),
-            ValueChange::ADD(1),
-            ValueChange::REMOVE(Some(1)),
+            ValueChange::Remove(Some(2)),
+            ValueChange::Add(1),
+            ValueChange::Remove(Some(1)),
         ],
     ));
     keeper.apply(&changes).unwrap();
@@ -266,87 +269,89 @@ fn test_aggregate_add_remove() {
 fn test_group_replace_remove() {
     let mut keeper = MockIndexKeeper::<u8, u8>::new();
     let mut changes = Vec::new();
-    changes.push(KeyChanges::new(2, vec![ValueChange::ADD(1), ValueChange::ADD(2)]));
+    changes.push(KeyChanges::new(2, vec![ValueChange::Add(1), ValueChange::Add(2)]));
     keeper.apply(&changes).unwrap();
-    assert_eq!(keeper.get(&2).ok(), Some(Some(Value::SINGLE(2))));
+    assert_eq!(keeper.get(&2).ok(), Some(Some(Value::Single(2))));
 
     let mut changes = Vec::new();
-    changes.push(KeyChanges::new(2, vec![ValueChange::REMOVE(Some(2))]));
+    changes.push(KeyChanges::new(2, vec![ValueChange::Remove(Some(2))]));
     keeper.apply(&changes).unwrap();
     assert_eq!(keeper.get(&2).ok(), Some(None));
 }
 
 #[test]
 fn test_group_values_remove() {
-    let mut keeper = MockIndexKeeper::<u8, u8>::new_mode(ValueMode::CLUSTER);
+    let mut keeper = MockIndexKeeper::<u8, u8>::new_mode(ValueMode::Cluster);
     let mut changes = Vec::new();
     changes.push(KeyChanges::new(
         2,
-        vec![ValueChange::ADD(1), ValueChange::ADD(2), ValueChange::ADD(3)],
+        vec![ValueChange::Add(1), ValueChange::Add(2), ValueChange::Add(3)],
     ));
     keeper.apply(&changes).unwrap();
-    assert_eq!(keeper.get(&2).ok(), Some(Some(Value::CLUSTER(vec![1, 2, 3]))));
+    assert_eq!(keeper.get(&2).ok(), Some(Some(Value::Cluster(vec![1, 2, 3]))));
 
     let mut changes = Vec::new();
     changes.push(KeyChanges::new(
         2,
-        vec![ValueChange::REMOVE(Some(1)), ValueChange::REMOVE(Some(2))],
+        vec![ValueChange::Remove(Some(1)), ValueChange::Remove(Some(2))],
     ));
     keeper.apply(&changes).unwrap();
-    assert_eq!(keeper.get(&2).ok(), Some(Some(Value::SINGLE(3))));
+    assert_eq!(keeper.get(&2).ok(), Some(Some(Value::Single(3))));
 }
 
 #[test]
 fn test_group_values_remove_no_order() {
-    let mut keeper = MockIndexKeeper::<u8, u8>::new_mode(ValueMode::CLUSTER);
+    let mut keeper = MockIndexKeeper::<u8, u8>::new_mode(ValueMode::Cluster);
     let mut changes = Vec::new();
     changes.push(KeyChanges::new(
         2,
-        vec![ValueChange::ADD(3), ValueChange::ADD(1), ValueChange::ADD(2)],
+        vec![ValueChange::Add(3), ValueChange::Add(1), ValueChange::Add(2)],
     ));
     keeper.apply(&changes).unwrap();
-    assert_eq!(keeper.get(&2).ok(), Some(Some(Value::CLUSTER(vec![1, 2, 3]))));
+    assert_eq!(keeper.get(&2).ok(), Some(Some(Value::Cluster(vec![1, 2, 3]))));
 
     let mut changes = Vec::new();
     changes.push(KeyChanges::new(
         2,
-        vec![ValueChange::REMOVE(Some(1)), ValueChange::REMOVE(Some(2))],
+        vec![ValueChange::Remove(Some(1)), ValueChange::Remove(Some(2))],
     ));
     keeper.apply(&changes).unwrap();
-    assert_eq!(keeper.get(&2).ok(), Some(Some(Value::SINGLE(3))));
+    assert_eq!(keeper.get(&2).ok(), Some(Some(Value::Single(3))));
 }
 
 #[test]
 fn test_add_same_value_twice() {
-    let mut keeper = MockIndexKeeper::<u8, u8>::new_mode(ValueMode::CLUSTER);
+    let mut keeper = MockIndexKeeper::<u8, u8>::new_mode(ValueMode::Cluster);
     let mut changes = Vec::new();
     changes.push(KeyChanges::new(
         2,
-        vec![ValueChange::ADD(1), ValueChange::ADD(2), ValueChange::ADD(1)],
+        vec![ValueChange::Add(1), ValueChange::Add(2), ValueChange::Add(1)],
     ));
 
-    changes.push(KeyChanges::new(1, vec![ValueChange::ADD(1), ValueChange::ADD(1)]));
+    changes.push(KeyChanges::new(1, vec![ValueChange::Add(1), ValueChange::Add(1)]));
     keeper.apply(&changes).unwrap();
-    assert_eq!(keeper.get(&2).ok(), Some(Some(Value::CLUSTER(vec![1, 2]))));
-    assert_eq!(keeper.get(&1).ok(), Some(Some(Value::SINGLE(1))));
+    assert_eq!(keeper.get(&2).ok(), Some(Some(Value::Cluster(vec![1, 2]))));
+    assert_eq!(keeper.get(&1).ok(), Some(Some(Value::Single(1))));
 }
 
 #[test]
 fn test_add_to_esclusive() {
-    let mut keeper = MockIndexKeeper::<u8, u8>::new_mode(ValueMode::EXCLUSIVE);
+    let mut keeper = MockIndexKeeper::<u8, u8>::new_mode(ValueMode::Exclusive);
     let mut changes = Vec::new();
-    changes.push(KeyChanges::new(2, vec![ValueChange::ADD(1)]));
+    changes.push(KeyChanges::new(2, vec![ValueChange::Add(1)]));
     keeper.apply(&changes).unwrap();
-    assert_eq!(keeper.get(&2).ok(), Some(Some(Value::SINGLE(1))));
+    assert_eq!(keeper.get(&2).ok(), Some(Some(Value::Single(1))));
     let mut changes = Vec::new();
-    changes.push(KeyChanges::new(2, vec![ValueChange::ADD(1)]));
+    changes.push(KeyChanges::new(2, vec![ValueChange::Add(1)]));
 
     keeper.apply(&changes).unwrap();
-    assert_eq!(keeper.get(&2).ok(), Some(Some(Value::SINGLE(1))));
+    assert_eq!(keeper.get(&2).ok(), Some(Some(Value::Single(1))));
     let mut changes = Vec::new();
-    changes.push(KeyChanges::new(2, vec![ValueChange::ADD(2)]));
+    changes.push(KeyChanges::new(2, vec![ValueChange::Add(2)]));
     assert!(match keeper.apply(&changes) {
-        Err(PersyError::IndexDuplicateKey(ref idxname, ref keyname)) if (idxname == "test_index" && keyname == "2") => {
+        Err(IndexChangeError::IndexDuplicateKey(ref idxname, ref keyname))
+            if (idxname == "test_index" && keyname == "2") =>
+        {
             true
         }
         _ => false,
@@ -355,14 +360,14 @@ fn test_add_to_esclusive() {
 
 #[test]
 fn test_group_key_remove() {
-    let mut keeper = MockIndexKeeper::<u8, u8>::new_mode(ValueMode::CLUSTER);
+    let mut keeper = MockIndexKeeper::<u8, u8>::new_mode(ValueMode::Cluster);
     let mut changes = Vec::new();
-    changes.push(KeyChanges::new(2, vec![ValueChange::ADD(1), ValueChange::ADD(2)]));
+    changes.push(KeyChanges::new(2, vec![ValueChange::Add(1), ValueChange::Add(2)]));
     keeper.apply(&changes).unwrap();
-    assert_eq!(keeper.get(&2).ok(), Some(Some(Value::CLUSTER(vec![1, 2]))));
+    assert_eq!(keeper.get(&2).ok(), Some(Some(Value::Cluster(vec![1, 2]))));
 
     let mut changes = Vec::new();
-    changes.push(KeyChanges::new(2, vec![ValueChange::REMOVE(None)]));
+    changes.push(KeyChanges::new(2, vec![ValueChange::Remove(None)]));
     keeper.apply(&changes).unwrap();
     assert_eq!(keeper.get(&2).ok(), Some(None));
 }
@@ -377,8 +382,8 @@ fn test_many_add_remove() {
     changes.sort_by_key(|k| k.k);
     keeper.apply(&changes).unwrap();
     print_tree(&mut keeper).unwrap();
-    assert_eq!(keeper.get(&2).ok(), Some(Some(Value::SINGLE(2))));
-    assert_eq!(keeper.get(&100).ok(), Some(Some(Value::SINGLE(100))));
+    assert_eq!(keeper.get(&2).ok(), Some(Some(Value::Single(2))));
+    assert_eq!(keeper.get(&100).ok(), Some(Some(Value::Single(100))));
     assert_eq!(keeper.get(&201).ok(), Some(None));
     let mut changes = Vec::new();
     for i in 0..200 {
@@ -404,7 +409,7 @@ fn test_many_add_remove_multiple_times() {
         }
         changes.sort_by_key(|k| k.k);
         keeper.apply(&changes).unwrap();
-        assert_eq!(keeper.get(&(n + 2)).ok(), Some(Some(Value::SINGLE(n + 2))));
+        assert_eq!(keeper.get(&(n + 2)).ok(), Some(Some(Value::Single(n + 2))));
         rchanges.sort_by_key(|k| k.k);
         keeper.apply(&rchanges).unwrap();
         assert_eq!(keeper.get(&(n + 2)).ok(), Some(None));
@@ -482,8 +487,8 @@ fn test_a_lot_add_remove_multiple_times() {
         }
         changes.sort_by_key(|k| k.k);
         keeper.apply(&changes).unwrap();
-        assert_eq!(keeper.get(&(2 * n)).ok(), Some(Some(Value::SINGLE(2 * n))));
-        assert_eq!(keeper.get(&(100 * n)).ok(), Some(Some(Value::SINGLE(100 * n))));
+        assert_eq!(keeper.get(&(2 * n)).ok(), Some(Some(Value::Single(2 * n))));
+        assert_eq!(keeper.get(&(100 * n)).ok(), Some(Some(Value::Single(100 * n))));
         assert_eq!(keeper.get(&20001).ok(), Some(None));
         remove.sort_by_key(|k| k.k);
         keeper.apply(&remove).unwrap();
@@ -505,8 +510,8 @@ fn test_big_tree() {
             }
         }
         keeper.apply(&changes).unwrap();
-        assert_eq!(keeper.get(&(2 * n)).ok(), Some(Some(Value::SINGLE(2 * n))));
-        assert_eq!(keeper.get(&(100 * n)).ok(), Some(Some(Value::SINGLE(100 * n))));
+        assert_eq!(keeper.get(&(2 * n)).ok(), Some(Some(Value::Single(2 * n))));
+        assert_eq!(keeper.get(&(100 * n)).ok(), Some(Some(Value::Single(100 * n))));
         assert_eq!(keeper.get(&20001).ok(), Some(None));
 
         keeper.apply(&remove).unwrap();
