@@ -5,9 +5,9 @@ use crate::{
     },
     allocator::Allocator,
     discref::{Page, PageOps},
-    error::{PRes, PersyError},
+    error::PERes,
     flush_checksum::double_buffer_check,
-    id::{PersyId, RecRef, SegmentId},
+    id::{RecRef, SegmentId},
     io::{read_u64, write_u64, InfallibleRead, InfallibleReadFormat, InfallibleWrite, InfallibleWriteFormat},
     persy::exp_from_content_size,
 };
@@ -46,7 +46,7 @@ impl Segment {
         }
     }
 
-    pub fn read(read: &mut dyn InfallibleRead) -> PRes<Segment> {
+    pub fn read(read: &mut dyn InfallibleRead) -> PERes<Segment> {
         let first_page = read.read_u64();
         let persistent_page = read.read_u64();
         let persistent_pos = read.read_u32();
@@ -75,7 +75,7 @@ impl Segment {
         write.write_all(self.name.as_bytes());
     }
 
-    pub fn allocate_internal(&mut self, allocator: &Allocator) -> PRes<(RecRef, Option<AllocatedSegmentPage>)> {
+    pub fn allocate_internal(&mut self, allocator: &Allocator) -> PERes<(RecRef, Option<AllocatedSegmentPage>)> {
         let page = self.last_page;
         let pos = self.last_pos;
         let new_pos = pos + ADDRESS_ENTRY_SIZE;
@@ -119,7 +119,7 @@ impl Segment {
         })
     }
 
-    pub fn collect_segment_pages(&self, allocator: &Allocator) -> PRes<Vec<u64>> {
+    pub fn collect_segment_pages(&self, allocator: &Allocator) -> PERes<Vec<u64>> {
         let mut pages = Vec::new();
         let mut page = self.first_page;
         let last = self.last_page;
@@ -150,21 +150,21 @@ impl Segment {
 }
 
 pub(crate) trait SegmentPageRead: PageOps {
-    fn segment_read_entry(&mut self, segment_id: SegmentId, pos: u32) -> PRes<Option<(u64, u16)>>;
-    fn segment_scan_all_entries(&mut self) -> PRes<(u64, [(u32, bool); ADDRESS_PAGE_ENTRY_COUNT as usize])>;
-    fn segment_first_available_pos(&mut self) -> PRes<u32>;
-    fn get_next(&mut self) -> PRes<u64>;
-    fn get_prev(&mut self) -> PRes<u64>;
-    fn empty(&mut self) -> PRes<bool>;
+    fn segment_read_entry(&mut self, segment_id: SegmentId, pos: u32) -> Option<(u64, u16)>;
+    fn segment_scan_all_entries(&mut self) -> (u64, [(u32, bool); ADDRESS_PAGE_ENTRY_COUNT as usize]);
+    fn segment_first_available_pos(&mut self) -> u32;
+    fn get_next(&mut self) -> PERes<u64>;
+    fn get_prev(&mut self) -> PERes<u64>;
+    fn empty(&mut self) -> PERes<bool>;
 }
 pub(crate) trait SegmentPage: SegmentPageRead {
-    fn segment_insert_entry(&mut self, segment_id: SegmentId, pos: u32, record_page: u64) -> PRes<()>;
-    fn segment_update_entry(&mut self, segment_id: SegmentId, pos: u32, record_page: u64) -> PRes<()>;
-    fn segment_delete_entry(&mut self, segment_id: SegmentId, pos: u32) -> PRes<bool>;
-    fn set_segment_id(&mut self, id: SegmentId) -> PRes<()>;
-    fn set_next(&mut self, next: u64) -> PRes<()>;
-    fn set_prev(&mut self, prev: u64) -> PRes<()>;
-    fn recalc_count(&mut self) -> PRes<()>;
+    fn segment_insert_entry(&mut self, segment_id: SegmentId, pos: u32, record_page: u64);
+    fn segment_update_entry(&mut self, segment_id: SegmentId, pos: u32, record_page: u64);
+    fn segment_delete_entry(&mut self, segment_id: SegmentId, pos: u32) -> bool;
+    fn set_segment_id(&mut self, id: SegmentId) -> PERes<()>;
+    fn set_next(&mut self, next: u64) -> PERes<()>;
+    fn set_prev(&mut self, prev: u64) -> PERes<()>;
+    fn recalc_count(&mut self) -> PERes<()>;
 }
 
 fn entry_exits(flags: u8) -> bool {
@@ -172,29 +172,24 @@ fn entry_exits(flags: u8) -> bool {
 }
 
 impl<T: InfallibleRead + PageOps> SegmentPageRead for T {
-    fn segment_read_entry(&mut self, segment_id: SegmentId, pos: u32) -> PRes<Option<(u64, u16)>> {
-        if pos > ADDRESS_PAGE_SIZE - ADDRESS_ENTRY_SIZE {
-            return Err(PersyError::InvalidId(
-                "Id page position too big than what expected".to_owned(),
-            ));
-        }
+    fn segment_read_entry(&mut self, segment_id: SegmentId, pos: u32) -> Option<(u64, u16)> {
         self.seek(SEGMENT_HASH_OFFSET);
         let persistent_id = SegmentId::read(self);
         if persistent_id != segment_id {
-            return Ok(None);
+            return None;
         }
         self.seek(pos);
         let record = self.read_u64();
         let flag = self.read_u8();
         let version = self.read_u16();
-        Ok(if !entry_exits(flag) || record == 0 {
+        if !entry_exits(flag) || record == 0 {
             None
         } else {
             Some((record, version))
-        })
+        }
     }
 
-    fn segment_scan_all_entries(&mut self) -> PRes<(u64, [(u32, bool); ADDRESS_PAGE_ENTRY_COUNT as usize])> {
+    fn segment_scan_all_entries(&mut self) -> (u64, [(u32, bool); ADDRESS_PAGE_ENTRY_COUNT as usize]) {
         let next_page = self.read_u64();
         let mut pos = SEGMENT_DATA_OFFSET;
         let mut recs = [(0, false); ADDRESS_PAGE_ENTRY_COUNT as usize];
@@ -209,10 +204,10 @@ impl<T: InfallibleRead + PageOps> SegmentPageRead for T {
                 break;
             }
         }
-        Ok((next_page, recs))
+        (next_page, recs)
     }
 
-    fn segment_first_available_pos(&mut self) -> PRes<u32> {
+    fn segment_first_available_pos(&mut self) -> u32 {
         let elements = (ADDRESS_PAGE_SIZE - SEGMENT_DATA_OFFSET) / ADDRESS_ENTRY_SIZE;
         let mut pos = SEGMENT_DATA_OFFSET + (elements - 1) * ADDRESS_ENTRY_SIZE;
         loop {
@@ -228,19 +223,19 @@ impl<T: InfallibleRead + PageOps> SegmentPageRead for T {
             pos -= ADDRESS_ENTRY_SIZE;
             debug_assert!(pos >= SEGMENT_DATA_OFFSET);
         }
-        Ok(pos)
+        pos
     }
 
-    fn get_next(&mut self) -> PRes<u64> {
+    fn get_next(&mut self) -> PERes<u64> {
         self.seek(0);
         Ok(self.read_u64())
     }
 
-    fn get_prev(&mut self) -> PRes<u64> {
+    fn get_prev(&mut self) -> PERes<u64> {
         self.seek(8);
         Ok(self.read_u64())
     }
-    fn empty(&mut self) -> PRes<bool> {
+    fn empty(&mut self) -> PERes<bool> {
         self.seek(SEGMENT_PAGE_DELETE_COUNT_OFFSET);
         Ok(self.read_u16() as u32 == ADDRESS_PAGE_ENTRY_COUNT)
     }
@@ -255,43 +250,37 @@ fn inc_version(version: u16) -> u16 {
 }
 
 impl<T: InfallibleRead + InfallibleWrite + PageOps> SegmentPage for T {
-    fn segment_insert_entry(&mut self, segment_id: SegmentId, pos: u32, record_page: u64) -> PRes<()> {
+    fn segment_insert_entry(&mut self, segment_id: SegmentId, pos: u32, record_page: u64) {
         debug_assert!(pos >= SEGMENT_DATA_OFFSET, "invalid page position {}", pos);
         self.seek(SEGMENT_HASH_OFFSET);
         let persistent_id = SegmentId::read(self);
-        if persistent_id != segment_id {
-            return Err(PersyError::SegmentNotFound);
-        }
+        debug_assert!(persistent_id == segment_id);
         // TODO: In case of restore this may get a wrong value, so on restore should be
         // re-calculated from the persistent flags.
         self.seek(pos);
         self.write_u64(record_page);
         self.write_u8(FLAG_EXISTS);
         self.write_u16(1);
-        Ok(())
     }
-    fn segment_update_entry(&mut self, segment_id: SegmentId, pos: u32, record_page: u64) -> PRes<()> {
+
+    fn segment_update_entry(&mut self, segment_id: SegmentId, pos: u32, record_page: u64) {
         debug_assert!(pos >= SEGMENT_DATA_OFFSET);
         self.seek(SEGMENT_HASH_OFFSET);
         let persistent_id = SegmentId::read(self);
-        if persistent_id != segment_id {
-            return Err(PersyError::RecordNotFound(PersyId(RecRef::new(self.get_index(), pos))));
-        }
+        debug_assert!(persistent_id == segment_id);
         self.seek(pos + 9);
         let version = self.read_u16();
         self.seek(pos);
         self.write_u64(record_page);
         self.seek(pos + 9);
         self.write_u16(inc_version(version));
-        Ok(())
     }
-    fn segment_delete_entry(&mut self, segment_id: SegmentId, pos: u32) -> PRes<bool> {
+
+    fn segment_delete_entry(&mut self, segment_id: SegmentId, pos: u32) -> bool {
         debug_assert!(pos >= SEGMENT_DATA_OFFSET);
         self.seek(SEGMENT_HASH_OFFSET);
         let persistent_id = SegmentId::read(self);
-        if persistent_id != segment_id {
-            return Err(PersyError::RecordNotFound(PersyId(RecRef::new(self.get_index(), pos))));
-        }
+        debug_assert!(persistent_id == segment_id);
         self.seek(SEGMENT_PAGE_DELETE_COUNT_OFFSET);
         let count = self.read_u16() + 1;
         self.seek(SEGMENT_PAGE_DELETE_COUNT_OFFSET);
@@ -300,28 +289,28 @@ impl<T: InfallibleRead + InfallibleWrite + PageOps> SegmentPage for T {
         let flag = self.read_u8();
         self.seek(pos + 8);
         self.write_u8(flag | FLAG_DELETED);
-        Ok(count as u32 == ADDRESS_PAGE_ENTRY_COUNT)
+        count as u32 == ADDRESS_PAGE_ENTRY_COUNT
     }
 
-    fn set_segment_id(&mut self, id: SegmentId) -> PRes<()> {
+    fn set_segment_id(&mut self, id: SegmentId) -> PERes<()> {
         self.seek(SEGMENT_HASH_OFFSET);
         id.write(self);
         Ok(())
     }
 
-    fn set_next(&mut self, next: u64) -> PRes<()> {
+    fn set_next(&mut self, next: u64) -> PERes<()> {
         self.seek(0);
         self.write_u64(next);
         Ok(())
     }
 
-    fn set_prev(&mut self, prev: u64) -> PRes<()> {
+    fn set_prev(&mut self, prev: u64) -> PERes<()> {
         self.seek(8);
         self.write_u64(prev);
         Ok(())
     }
 
-    fn recalc_count(&mut self) -> PRes<()> {
+    fn recalc_count(&mut self) -> PERes<()> {
         let mut pos = SEGMENT_DATA_OFFSET;
         let mut count = 0;
         loop {
@@ -363,7 +352,7 @@ pub fn segment_hash(segment: &str) -> u64 {
 }
 
 impl Segments {
-    pub fn new(root_page: u64, allocator: &Arc<Allocator>) -> PRes<Segments> {
+    pub fn new(root_page: u64, allocator: &Arc<Allocator>) -> PERes<Segments> {
         let mut buffer_0 = [0; 19];
         let mut buffer_1 = [0; 19];
         let page_id;
@@ -418,7 +407,7 @@ impl Segments {
         })
     }
 
-    pub fn init(root: Page, allocator: &Allocator) -> PRes<()> {
+    pub fn init(root: Page, allocator: &Allocator) -> PERes<()> {
         let mut buffer = [0; 19];
         write_u64(&mut buffer[0..8], 0);
         write_u64(&mut buffer[8..16], 0);
@@ -450,7 +439,7 @@ impl Segments {
         self.segments_id.contains_key(segment)
     }
 
-    pub fn create_temp_segment(&mut self, allocator: &Allocator, segment: &str) -> PRes<(SegmentId, u64)> {
+    pub fn create_temp_segment(&mut self, allocator: &Allocator, segment: &str) -> PERes<(SegmentId, u64)> {
         let mut allocated = allocator.allocate(ADDRESS_PAGE_EXP)?;
         let allocated_id = allocated.get_index();
         let segment_id = SegmentId::new(segment_hash(segment));
@@ -468,7 +457,7 @@ impl Segments {
         self.temp_segments.get_mut(&segment)
     }
 
-    pub fn drop_temp_segment(&mut self, allocator: &Allocator, segment: SegmentId) -> PRes<()> {
+    pub fn drop_temp_segment(&mut self, allocator: &Allocator, segment: SegmentId) -> PERes<()> {
         if let Some(segment) = self.temp_segments.remove(&segment) {
             self.temp_segments_id.remove(&segment.name);
             //TODO: to review this, may cause disc leaks in case of crash on rollback of segment
@@ -485,7 +474,7 @@ impl Segments {
         self.segments.contains_key(&segment) || self.temp_segments.contains_key(&segment)
     }
 
-    pub fn create_segment(&mut self, segment: SegmentId, first_page: u64) -> PRes<()> {
+    pub fn create_segment(&mut self, segment: SegmentId, first_page: u64) {
         if let Some(mut s) = self.temp_segments.remove(&segment) {
             // This is needed mainly for recover
             s.first_page = first_page;
@@ -493,17 +482,15 @@ impl Segments {
             self.segments_id.insert(s.name.clone(), s.segment_id);
             self.segments.insert(s.segment_id, s);
         }
-        Ok(())
     }
 
-    pub fn drop_segment(&mut self, segment: &str) -> PRes<()> {
+    pub fn drop_segment(&mut self, segment: &str) {
         if let Some(seg) = self.segments_id.remove(segment) {
             self.segments.remove(&seg);
         }
-
-        Ok(())
     }
-    pub fn collect_segment_pages(&self, allocator: &Allocator, segment: SegmentId) -> PRes<Vec<u64>> {
+
+    pub fn collect_segment_pages(&self, allocator: &Allocator, segment: SegmentId) -> PERes<Vec<u64>> {
         if let Some(seg) = self.segments.get(&segment) {
             seg.collect_segment_pages(allocator)
         } else {
@@ -511,14 +498,14 @@ impl Segments {
         }
     }
 
-    pub fn set_first_page(&mut self, segment: SegmentId, first_page: u64) -> PRes<()> {
+    pub fn set_first_page(&mut self, segment: SegmentId, first_page: u64) -> PERes<()> {
         if let Some(seg) = self.segments.get_mut(&segment) {
             seg.first_page = first_page;
         }
         Ok(())
     }
 
-    pub fn confirm_allocations(&mut self, segments: &[SegmentId], allocator: &Allocator, recover: bool) -> PRes<()> {
+    pub fn confirm_allocations(&mut self, segments: &[SegmentId], allocator: &Allocator, recover: bool) -> PERes<()> {
         for seg in segments {
             // If happen that a segment is not found something went wrong and is better crash
             let segment = if let Some(s) = self.segments.get_mut(seg) {
@@ -531,13 +518,13 @@ impl Segments {
             if recover {
                 // In case of recover recalculate the last available position
                 let mut page = allocator.load_page(segment.last_page)?;
-                segment.last_pos = page.segment_first_available_pos()?;
+                segment.last_pos = page.segment_first_available_pos();
             }
         }
         Ok(())
     }
 
-    pub fn flush_segments(&mut self, allocator: &Allocator) -> PRes<()> {
+    pub fn flush_segments(&mut self, allocator: &Allocator) -> PERes<()> {
         let mut buffer = Vec::<u8>::new();
         for segment in self.segments.values() {
             buffer.write_u8(1);
@@ -677,11 +664,11 @@ mod tests {
         let mut segments = Segments::new(root_index, &all).unwrap();
 
         let (id, fp) = segments.create_temp_segment(&all, "some").unwrap();
-        segments.create_segment(id, fp).unwrap();
+        segments.create_segment(id, fp);
         segments.flush_segments(&all).unwrap();
         assert!(segments.segments_id.contains_key("some"));
         assert!(segments.segments.contains_key(&id));
-        segments.drop_segment("some").unwrap();
+        segments.drop_segment("some");
         segments.flush_segments(&all).unwrap();
         assert!(!segments.segments_id.contains_key("some"));
         assert!(!segments.segments.contains_key(&id));
@@ -716,7 +703,7 @@ mod tests {
             let mut segments = Segments::new(root_index, &all).unwrap();
             let (c_id, fp) = segments.create_temp_segment(&all, "some").unwrap();
             id = c_id;
-            segments.create_segment(id, fp).unwrap();
+            segments.create_segment(id, fp);
             segments.flush_segments(&all).unwrap();
         }
         {
@@ -724,7 +711,7 @@ mod tests {
             assert_eq!(segments.segments.len(), 1);
             assert!(segments.segments_id.contains_key("some"));
             assert!(segments.segments.contains_key(&id));
-            segments.drop_segment("some").unwrap();
+            segments.drop_segment("some");
             segments.flush_segments(&all).unwrap();
         }
         {
@@ -745,7 +732,7 @@ mod tests {
             let mut segments = Segments::new(root_index, &all).unwrap();
             for i in 0..100 {
                 let (id, fp) = segments.create_temp_segment(&all, &format!("some{}", i)).unwrap();
-                segments.create_segment(id, fp).unwrap();
+                segments.create_segment(id, fp);
             }
             segments.flush_segments(&all).unwrap();
         }
@@ -753,7 +740,7 @@ mod tests {
             let mut segments = Segments::new(root_index, &all).unwrap();
             for i in 0..100 {
                 assert!(segments.segments_id.contains_key(&format!("some{}", i)));
-                segments.drop_segment(&format!("some{}", i)).unwrap();
+                segments.drop_segment(&format!("some{}", i));
             }
             segments.flush_segments(&all).unwrap();
         }
@@ -769,8 +756,8 @@ mod tests {
     fn test_seg_insert_read_pointer() {
         let mut page = Page::new(vec![0; 1024], 0, 0, 10);
         let segment_id = SegmentId::new(0);
-        page.segment_insert_entry(segment_id, 30, 10).unwrap();
-        let read = page.segment_read_entry(segment_id, 30).unwrap();
+        page.segment_insert_entry(segment_id, 30, 10);
+        let read = page.segment_read_entry(segment_id, 30);
         match read {
             Some(val) => assert_eq!(val.0, 10),
             None => assert!(false),
@@ -781,9 +768,9 @@ mod tests {
     fn test_seg_insert_update_read_pointer() {
         let mut page = Page::new(vec![0; 1024], 0, 0, 10);
         let segment_id = SegmentId::new(0);
-        page.segment_insert_entry(segment_id, 30, 10).unwrap();
-        page.segment_update_entry(segment_id, 30, 15).unwrap();
-        let read = page.segment_read_entry(segment_id, 30).unwrap();
+        page.segment_insert_entry(segment_id, 30, 10);
+        page.segment_update_entry(segment_id, 30, 15);
+        let read = page.segment_read_entry(segment_id, 30);
         match read {
             Some(val) => assert_eq!(val.0, 15),
             None => assert!(false),
@@ -794,9 +781,9 @@ mod tests {
     fn test_seg_insert_delete_read_pointer() {
         let mut page = Page::new(vec![0; 1024], 0, 0, 10);
         let segment_id = SegmentId::new(0);
-        page.segment_insert_entry(segment_id, 30, 10).unwrap();
-        page.segment_delete_entry(segment_id, 30).unwrap();
-        let read = page.segment_read_entry(segment_id, 30).unwrap();
+        page.segment_insert_entry(segment_id, 30, 10);
+        page.segment_delete_entry(segment_id, 30);
+        let read = page.segment_read_entry(segment_id, 30);
         match read {
             Some(_) => assert!(false),
             None => assert!(true),
